# -*- coding: utf-8 -*-

from datetime import datetime
from imio.pm.ws.config import POD_TEMPLATE_ID_PATTERN
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import cleanMemoize
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from plone import api
from Products.Archetypes.event import ObjectEditedEvent
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter
from zope.event import notify

import transaction


class testRESTMethods(WS4PMCLIENTTestCase):
    """
        Tests the browser.settings REST client methods
    """

    def test_rest_connectToPloneMeeting(self):
        """Check that we can actually connect to PloneMeeting with given parameters."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        settings = ws4pmSettings.settings()
        setCorrectSettingsConfig(self.portal, minimal=True)
        # with valid informations, we can connect to PloneMeeting SOAP webservices
        self.failUnless(ws4pmSettings._rest_connectToPloneMeeting())
        # if either url or username/password is not valid, we can not connect...
        valid_url = settings.pm_url
        settings.pm_url = settings.pm_url + 'invalidEndOfURL'
        cleanMemoize(self.request)
        # with invalid url, it fails...
        self.failIf(ws4pmSettings._rest_connectToPloneMeeting())
        settings.pm_url = valid_url
        # with valid url but wrong password, it fails...
        settings.pm_password = u'wrongPassword'
        cleanMemoize(self.request)
        self.failIf(ws4pmSettings._rest_connectToPloneMeeting())

    def test_rest_checkIsLinked(self):
        """Verify that we can get link informations"""
        ws4pmSettings = getMultiAdapter(
            (self.portal, self.request),
            name="ws4pmclient-settings",
        )
        setCorrectSettingsConfig(self.portal, minimal=True)

        cfg = self.meetingConfig
        self._removeConfigObjectsFor(cfg)

        self.changeUser("pmCreator1")
        item1 = self.create("MeetingItem")
        item1.externalIdentifier = "external1"
        item1.reindexObject(idxs=["externalIdentifier", ])
        transaction.commit()

        data = {
            "UID": item1.UID(),
            "externalIdentifier": "external1",
        }
        result = ws4pmSettings._rest_checkIsLinked(data)
        self.assertEqual(item1.UID(), result["UID"])
        self.assertEqual(0, result["extra_include_linked_items_items_total"])

        cfg.setItemManualSentToOtherMCStates(("itemcreated", ))
        self.changeUser("pmCreator2")
        item2 = self.create("MeetingItem", decision="My Decision")
        item2.externalIdentifier = "external2"
        item2.reindexObject(idxs=["externalIdentifier", ])
        self.changeUser("pmManager")
        meeting = self.create("Meeting")
        self.presentItem(item2)
        self.decideMeeting(meeting)
        self.do(item2, "delay")
        transaction.commit()

        item2_link = item2.get_successors()[0]

        data = {
            "UID": item2.UID(),
            "externalIdentifier": "external2",
        }
        result = ws4pmSettings._rest_checkIsLinked(data)
        self.assertEqual(item2.UID(), result["UID"])
        self.assertEqual(1, result["extra_include_linked_items_items_total"])
        self.assertEqual(
            item2_link.UID(),
            result["extra_include_linked_items"][0]["UID"],
        )

    def test_rest_getConfigInfos(self):
        """Check that we receive valid infos about the PloneMeeting's configuration."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        configInfos = ws4pmSettings._rest_getConfigInfos()
        # check that we received meeting config elements
        self.assertTrue(len(configInfos), 2)
        self.assertTrue(configInfos[0]['id'], u'plonemeeting-assembly')
        self.assertTrue(configInfos[1]['id'], u'plonegov-assembly')
        # by default, no categories
        # plonemeeting-assembly has no categories BTW, so we use plonegov-assembly
        self.assertFalse(configInfos[1].get('categories', False))
        # we can ask categories by passing a showCategories=True to _rest_getConfigInfos
        configInfos = ws4pmSettings._rest_getConfigInfos(showCategories=True)
        self.assertEqual(len(configInfos[1].get('categories')), 6)

    def test_rest_getItemCreationAvailableData(self):
        """Check that we receive the list of available data for creating an item."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)

        availableData = set(ws4pmSettings._rest_getItemCreationAvailableData())
        self.assertEqual(availableData, {u'annexes',
                                         u'associatedGroups',
                                         u'category',
                                         u'copyGroups',
                                         u'decision',
                                         u'description',
                                         u'externalIdentifier',
                                         u'extraAttrs',
                                         u'groupsInCharge',
                                         u'ignore_validation_for',
                                         u'ignore_not_used_data',
                                         u'motivation',
                                         u'optionalAdvisers',
                                         u'preferredMeeting',
                                         u'proposingGroup',
                                         u'title',
                                         u'toDiscuss'})

    def test_rest_getItemInfos(self):
        """Check the fact of getting informations about an existing item."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        # by default no item exist in the portal...  So create one!
        self.changeUser('pmManager')
        item = self.create('MeetingItem')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        self.assertTrue(len(ws4pmSettings._rest_getItemInfos({'UID': item.UID()})) == 1)
        # getItemInfos is called inTheNameOf the currently connected user
        # if the user (like 'pmCreator1') can see the item, he gets it in the request
        # either (like for 'pmCreator2') the item is not found
        self.changeUser('pmCreator1')
        self.assertTrue(len(ws4pmSettings._rest_getItemInfos({'UID': item.UID()})) == 1)
        self.changeUser('pmCreator2')
        self.assertTrue(len(ws4pmSettings._rest_getItemInfos({'UID': item.UID()})) == 0)

    def test_rest_getUserInfos_groups_and_suffix(self):
        """Test _rest_getUserInfos with group and suffix parameter"""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name="ws4pmclient-settings")
        setCorrectSettingsConfig(self.portal, minimal=True)
        self.changeUser("pmCreator1")

        user_infos = ws4pmSettings._rest_getUserInfos(
            showGroups=True,
            suffix="creators",
        )
        self.assertIsNotNone(user_infos)
        self.assertEqual("pmCreator1", user_infos["id"])
        self.assertEqual(1, user_infos["extra_include_groups_items_total"])
        self.assertEqual("developers", user_infos["extra_include_groups"][0]["id"])

        # Suffix that does not match any group
        user_infos = ws4pmSettings._rest_getUserInfos(
            showGroups=True,
            suffix="observers",
        )
        self.assertIsNotNone(user_infos)
        self.assertEqual("pmCreator1", user_infos["id"])
        self.assertEqual(0, user_infos["extra_include_groups_items_total"])

    def test_rest_getUserInfos_groups_and_wihout_suffix(self):
        """Test _rest_getUserInfos with only group parameter"""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name="ws4pmclient-settings")
        setCorrectSettingsConfig(self.portal, minimal=True)
        self.changeUser("pmCreator1")

        user_infos = ws4pmSettings._rest_getUserInfos(
            showGroups=True,
        )
        self.assertIsNotNone(user_infos)
        self.assertEqual("pmCreator1", user_infos["id"])
        self.assertEqual(1, user_infos["extra_include_groups_items_total"])
        self.assertEqual("developers", user_infos["extra_include_groups"][0]["id"])

    def test_rest_getUserInfos_default(self):
        """Test _rest_getUserInfos with default parameters (no groups or suffix)"""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name="ws4pmclient-settings")
        setCorrectSettingsConfig(self.portal, minimal=True)
        self.changeUser("pmCreator1")

        user_infos = ws4pmSettings._rest_getUserInfos()
        self.assertIsNotNone(user_infos)
        self.assertEqual("pmCreator1", user_infos["id"])
        self.assertTrue("extra_include_groups" not in user_infos)

    def test_rest_searchItems(self):
        """Check the fact of searching items informations about existing items."""
        SAME_TITLE = 'sameTitleForBothItems'
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        # Create 2 items, one for 'pmCreator1' and one for 'pmCreator2'
        # items are only viewable by their creator as 'pmCreatorx' not in the same proposingGroup
        self.changeUser('pmCreator1')
        item1 = self.create('MeetingItem')
        item1.setTitle(SAME_TITLE)
        item1.reindexObject(idxs=['Title', ])
        self.changeUser('pmCreator2')
        item2 = self.create('MeetingItem')
        item2.setTitle(SAME_TITLE)
        item2.reindexObject(idxs=['Title', ])
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        # searchItems will automatically restrict searches to the connected user
        self.changeUser('pmCreator1')
        result = ws4pmSettings._rest_searchItems({'Title': SAME_TITLE})
        self.assertTrue(len(result), 1)
        self.assertTrue(result[0]["UID"] == item1.UID())
        self.changeUser('pmCreator2')
        result = ws4pmSettings._rest_searchItems({'Title': SAME_TITLE})
        self.assertTrue(len(result), 1)
        self.assertTrue(result[0]["UID"] == item2.UID())

    def test_rest_createItem_with_ignore_validation(self):
        """Check item creation with `ignore_validation_for` parameter"""
        cfg2 = self.meetingConfig2
        cfg2Id = cfg2.getId()
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        self.changeUser('pmManager')
        self.setMeetingConfig(cfg2Id)
        self._enableField("internalNotes")
        test_meeting = self.create('Meeting')
        self.freezeMeeting(test_meeting)
        self.changeUser('pmCreator1')
        # create the 'pmCreator1' member area to be able to create an item
        pmFolder = self.tool.getPloneMeetingFolder(cfg2Id)
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        # create an item for 'pmCreator1'
        data = {'title': u'My sample item',
                'description': u'<p>My description</p>',
                # also use accents, this was failing with suds-jurko 0.5
                'decision': u'<p>My d\xe9cision</p>',
                'preferredMeeting': test_meeting.UID(),
                'externalIdentifier': u'my-external-identifier',
                'extraAttrs': [{'key': 'internalNotes',
                                'value': '<p>Internal notes</p>'}]}
        result = ws4pmSettings._rest_createItem(cfg2Id, 'developers', data)
        self.assertIsNone(result)
        messages = IStatusMessage(self.request)
        self.assertEqual(
            messages.show()[-1].message,
            u"An error occured during the item creation in PloneMeeting! The error "
            "message was : [{'field': 'category', 'message': u'Please select a "
            "category.', 'error': 'ValidationError'}]"
        )

        data["ignore_validation_for"] = "category"
        result = ws4pmSettings._rest_createItem(cfg2Id, 'developers', data)
        # commit again so the item is really created
        transaction.commit()
        # the item is created and his UID is returned
        # check that the item is actually created inTheNameOf 'pmCreator1'
        self.assertIsNotNone(result)
        itemUID = result[0]
        item = self.portal.uid_catalog(UID=itemUID)[0].getObject()
        self.assertTrue(item.aq_inner.aq_parent.UID(), pmFolder.UID())
        self.assertTrue(item.owner_info()['id'] == 'pmCreator1')
        self.assertEqual(item.Title(), data['title'])

    def test_rest_createItem(self):
        """Check item creation.
           Item creation will automatically use currently connected user
           to create the item regarding the _getUserIdToUseInTheNameOfWith."""
        cfg2 = self.meetingConfig2
        cfg2Id = cfg2.getId()
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        self.changeUser('pmManager')
        self.setMeetingConfig(cfg2Id)
        self._enableField("internalNotes")
        test_meeting = self.create('Meeting')
        self.freezeMeeting(test_meeting)
        self.changeUser('pmCreator1')
        # create the 'pmCreator1' member area to be able to create an item
        pmFolder = self.tool.getPloneMeetingFolder(cfg2Id)
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        # create an item for 'pmCreator1'
        data = {'title': u'My sample item',
                'category': u'deployment',
                'description': u'<p>My description</p>',
                # also use accents, this was failing with suds-jurko 0.5
                'decision': u'<p>My d\xe9cision</p>',
                'preferredMeeting': test_meeting.UID(),
                'externalIdentifier': u'my-external-identifier',
                'extraAttrs': [{'key': 'internalNotes',
                                'value': '<p>Internal notes</p>'}]}
        result = ws4pmSettings._rest_createItem(cfg2Id, 'developers', data)
        # commit again so the item is really created
        transaction.commit()
        # the item is created and his UID is returned
        # check that the item is actually created inTheNameOf 'pmCreator1'
        self.assertIsNotNone(result)
        itemUID = result[0]
        item = self.portal.uid_catalog(UID=itemUID)[0].getObject()
        # created in the 'pmCreator1' member area
        self.assertTrue(item.aq_inner.aq_parent.UID(), pmFolder.UID())
        self.assertTrue(item.owner_info()['id'] == 'pmCreator1')
        self.assertEqual(item.Title(), data['title'])
        self.assertEqual(item.getCategory(), data['category'])
        self.assertEqual(item.Description(), data['description'])
        self.assertEqual(item.getDecision(), data['decision'].encode('utf-8'))
        self.assertEqual(item.getPreferredMeeting(), test_meeting.UID(), data['preferredMeeting'])
        self.assertEqual(item.externalIdentifier, data['externalIdentifier'])
        # extraAttrs
        # XXX Does not work ATM, value is always empty
        # self.assertEqual(item.getInternalNotes(), data['internalNotes'])

        # if we try to create with wrong data, the SOAP ws returns a response
        # that is displayed to the user creating the item
        data['category'] = 'unexisting-category-id'
        result = ws4pmSettings._rest_createItem('plonegov-assembly', 'developers', data)
        self.assertIsNone(result)
        messages = IStatusMessage(self.request)
        # a message is displayed
        self.assertEqual(messages.show()[-1].message,
                         u"An error occured during the item creation in PloneMeeting! "
                         "The error message was : [{'field': 'category', 'message': "
                         "u'Please select a category.', 'error': 'ValidationError'}]")

    def test_rest_getItemTemplate(self):
        """Check while getting rendered template for an item.
           getItemTemplate will automatically use currently connected user
           to render item template regarding the _getUserIdToUseInTheNameOfWith."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        # by default no item exist in the portal...  So create one!
        self.changeUser('pmManager')
        item = self.create('MeetingItem')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        self.assertTrue(ws4pmSettings._rest_getItemTemplate(
            {'itemUID': item.UID(),
             'templateId': POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt')}))
        # getItemTemplate is called inTheNameOf the currently connected user
        # if the user (like 'pmCreator1') can see the item, he gets the rendered template
        # either (like for 'pmCreator2') nothing is returned
        self.changeUser('pmCreator1')
        self.assertTrue(ws4pmSettings._rest_getItemTemplate(
            {'itemUID': item.UID(),
             'templateId': POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt')}))
        self.changeUser('pmCreator2')
        self.assertFalse(ws4pmSettings._rest_getItemTemplate(
            {'itemUID': item.UID(),
             'templateId': POD_TEMPLATE_ID_PATTERN.format('itemTemplate', 'odt')}))

    def test_rest_getMeetingAcceptingItems(self):
        """Check getting accepting items meeting.
           Should only return meetings in the state 'creation' and 'frozen'"""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal, minimal=True)
        cfg = self.meetingConfig
        cfgId = cfg.getId()
        meetings = ws4pmSettings._rest_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        self.assertEqual(meetings, [])
        self.changeUser('pmManager')
        meeting_1 = self.create('Meeting', date=datetime(2013, 3, 3))
        meeting_2 = self.create('Meeting', date=datetime(2013, 3, 3))
        transaction.commit()
        self.changeUser('pmCreator1')
        meetings = ws4pmSettings._rest_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        # so far find the two meetings
        self.assertEqual(len(meetings), 2)

        # freeze meeting_2 => it still should be in the accepting items meetings
        self.changeUser('pmManager')
        api.content.transition(meeting_2, 'freeze')
        self.portal._volatile_cache_keys._p_changed = False  # Avoid ConflictError
        transaction.commit()
        self.changeUser('pmCreator1')
        meetings = ws4pmSettings._rest_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        self.assertEqual(len(meetings), 2)

        self.changeUser('pmManager')
        api.content.transition(meeting_2, 'publish')
        api.content.transition(meeting_2, 'decide')
        transaction.commit()
        # after publishing meeting_2, it should not be in the results anymore.
        self.changeUser('pmCreator1')
        meetings = ws4pmSettings._rest_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmCreator1'}
        )
        self.assertEqual(len(meetings), 1)
        self.assertEqual(meetings[0]["UID"], meeting_1.UID())
        # if no inTheNameOf param is explicitly passed, _rest_getMeetingsAcceptingItems()
        # should set a default one.
        meetings = ws4pmSettings._rest_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId}
        )
        self.assertEqual(len(meetings), 1)
        self.assertEqual(meetings[0]["UID"], meeting_1.UID())

        # As pmManager, we should get all the meetings
        meetings = ws4pmSettings._rest_getMeetingsAcceptingItems(
            {'meetingConfigId': cfgId, 'inTheNameOf': 'pmManager'}
        )
        self.assertEqual(len(meetings), 2)

        # Ensure that meetings have a date
        self.assertEqual("2013-03-03T00:00:00", meetings[0]["date"])

    def test_rest_getDecidedMeetingDate(self):
        """Test the _rest_getDecidedMeetingDate method that should return
        the date of which the item has been actually decided."""
        setCorrectSettingsConfig(self.portal, minimal=True)
        cfg = self.meetingConfig
        cfg2= self.meetingConfig2
        cfg2Id = cfg2.getId()
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')

        self.changeUser('admin')
        self._activate_wfas(('delayed', 'postpone_next_meeting', ))
        # Setting up sending to another meetingConfig
        cfg.setMeetingConfigsToCloneTo(({'meeting_config': '%s' % cfg2Id,
                                         'trigger_workflow_transitions_until': '%s.%s' %
                                         (cfg2Id, 'validated')},))
        cfg.setItemAutoSentToOtherMCStates((u'accepted', ))
        notify(ObjectEditedEvent(cfg))
        self.changeUser("pmManager")
        # Situation 1: item 'item_delayed' sent to PM will be delayed in a meeting 'meeting_delayed'
        # It will be replaced by 'item_decided' and decided in a next meeting 'meeting_decided'
        # It will also be sent to meetingConfig2
        meeting_delayed = self.create('Meeting', date=datetime(2024, 7, 20))
        item_delayed = self.create('MeetingItem', decision="A Decision", externalIdentifier=u'external1')
        item_delayed.setOtherMeetingConfigsClonableTo((cfg2Id,))
        item_delayed.reindexObject(idxs=['sentToInfos', "externalIdentifier",])
        self.presentItem(item_delayed)
        self.decideMeeting(meeting_delayed)
        self.do(item_delayed, 'delay')
        item_decided = item_delayed.get_successor()
        meeting_decided_date = datetime(2024, 7, 21)
        meeting_decided = self.create('Meeting', date=meeting_decided_date)
        self.presentItem(item_decided)
        self.decideMeeting(meeting_decided)
        self.do(item_decided, 'accept')
        transaction.commit()
        self.assertEqual(
            ws4pmSettings._rest_getDecidedMeetingDate(
                {'externalIdentifier': 'external1', 'inTheNameOf': 'pmManager'},
                item_portal_type=cfg.getItemTypeName()
            ),
            meeting_decided_date
        )
        # Situation 2: 'item_decided' has been sent to meetingConfig2 as an other item 'item_sent'
        # and will be decided there too. We want the date of the meeting `meeting2_decided`
        # in which 'item_sent' is decided.
        item_sent = item_decided.get_successor()
        item_sent.category = 'deployment' # categories are activated in cfg2
        self.setMeetingConfig(cfg2Id)
        meeting2_decided_date = datetime(2024, 7, 22)
        meeting2_decided = self.create('Meeting', date=meeting2_decided_date)
        self.presentItem(item_sent)
        self.decideMeeting(meeting2_decided)
        self.do(item_sent, 'accept')
        transaction.commit()
        self.assertEqual(
            ws4pmSettings._rest_getDecidedMeetingDate(
                {'externalIdentifier': 'external1', 'inTheNameOf': 'pmManager'},
                item_portal_type=cfg2.getItemTypeName()
            ),
            meeting2_decided_date
        )
        # It should still give us the meeting_decided_date for situation 1
        self.assertEqual(
            ws4pmSettings._rest_getDecidedMeetingDate(
                {'externalIdentifier': 'external1', 'inTheNameOf': 'pmManager'},
                item_portal_type=cfg.getItemTypeName()
            ),
            meeting_decided_date
        )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testRESTMethods, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-
#
# File: testItemMethods.py
#
# GNU General Public License (GPL)
#

from AccessControl import Unauthorized
from imio.pm.wsclient.config import ACTION_SUFFIX
from imio.pm.wsclient.testing import WS4PMCLIENT_PM_TESTING_PROFILE_FUNCTIONAL
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing.interfaces import TEST_USER_ID
from plone.app.testing.interfaces import TEST_USER_NAME
from zope.component import getMultiAdapter
from zope.event import notify
from zope.tales.tales import CompilerError
from zope.traversing.interfaces import BeforeTraverseEvent

import unittest


class testSettings(unittest.TestCase):
    """
        Tests the browser.settings SOAP client methods
    """

    layer = WS4PMCLIENT_PM_TESTING_PROFILE_FUNCTIONAL

    def setUp(self):
        portal = self.layer['portal']
        request = self.layer['request']
        self.portal = portal
        self.request = request
        # setup manually the correct browserlayer, see:
        # https://dev.plone.org/ticket/11673
        notify(BeforeTraverseEvent(self.portal, self.request))

    def test_ws4pmSettings(self):
        """Check that we can actually access settings and that we have the correct fields."""
        # settings are only available to connected users having "Manage portal" permission
        self.assertRaises(Unauthorized, self.portal.restrictedTraverse, '@@ws4pmclient-settings')
        login(self.portal, TEST_USER_NAME)
        self.assertRaises(Unauthorized, self.portal.restrictedTraverse, '@@ws4pmclient-settings')
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        ws4pmSettings = self.portal.restrictedTraverse('@@ws4pmclient-settings')
        settings = ws4pmSettings.settings()
        fields = settings.__schema__._InterfaceClass__attrs.keys()
        fields.sort()
        self.assertEquals(fields, ['allowed_annexes_types',
                                   'field_mappings',
                                   'generated_actions',
                                   'only_one_sending',
                                   'pm_password',
                                   'pm_timeout',
                                   'pm_url',
                                   'pm_username',
                                   'user_mappings',
                                   'viewlet_display_condition'])

    def test_saveSettings(self):
        """While settings are saved, some actions are added to portal_actions/object_buttons."""
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        login(self.portal, TEST_USER_NAME)
        # for now, there are no relative plonemeeting actions in portal_actions/object_buttons
        object_buttons_ids = self.portal.portal_actions.object_buttons.objectIds()
        self.failIf([actId for actId in object_buttons_ids if actId.startswith(ACTION_SUFFIX)])
        setCorrectSettingsConfig(self.portal, withValidation=False)
        # now relevant actions exist
        object_buttons_ids = self.portal.portal_actions.object_buttons.objectIds()
        self.assertEquals(len([actId for actId in object_buttons_ids if actId.startswith(ACTION_SUFFIX)]), 5)
        # and it is correctly configured
        setRoles(self.portal, TEST_USER_ID, ('Member',))
        # plone.app.testing does not manage request/URL and request/ACTUAL_URL
        # and request/ACTUAL_URL is necessary for listFilteredActionsFor
        self.request.set('URL', self.portal.absolute_url())
        self.request.set('ACTUAL_URL', self.portal.absolute_url())
        object_buttons = self.portal.portal_actions.listFilteredActionsFor(self.portal)['object_buttons']
        # 2 of the generated actions are not available to non 'Managers'
        self.assertEquals(len([act for act in object_buttons if act['id'].startswith(ACTION_SUFFIX)]), 5 - 2)
        # now save again with just 2 actions to generate
        generated_actions = [
            {'pm_meeting_config_id': 'plonegov-assembly',
             'condition': u'python:True',
             'permissions': u'View'},
            {'pm_meeting_config_id': 'plonemeeting-assembly',
             'condition': u'python:True',
             'permissions': u'View'},
        ]
        setCorrectSettingsConfig(self.portal,
                                 withValidation=False,
                                 **{'generated_actions': generated_actions})
        object_buttons = self.portal.portal_actions.listFilteredActionsFor(self.portal)['object_buttons']
        pm_object_buttons = [act for act in object_buttons if act['id'].startswith(ACTION_SUFFIX)]
        # only 2 actions exist now
        self.assertEquals(len(pm_object_buttons), 2)
        # and it is valid ones
        self.assertTrue('meetingConfigId=plonegov-assembly' in pm_object_buttons[0]['url'])
        self.assertTrue('meetingConfigId=plonemeeting-assembly' in pm_object_buttons[1]['url'])

    def test_getUserIdToUseInTheNameOfWith(self):
        """Returns the userId that will actually create the item.
           Returns None if we found out that it is the defined settings.pm_username
           that will create the item : either it is the currently connected user,
           or there is an existing user_mapping between currently connected user
           and settings.pm_username user. """
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        login(self.portal, TEST_USER_NAME)
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        # this method is taking care of values in settings.pm_username
        # and settings.user_mappings
        # first define no user_mappings and no pm_username,
        # the currently logged in user will be the creator
        self.assertEquals(ws4pmSettings._getUserIdToUseInTheNameOfWith(), TEST_USER_ID)
        # check that if current member is settings.pm_username, None is returned
        settings = ws4pmSettings.settings()
        settings.pm_username = unicode(TEST_USER_ID, 'utf-8')
        self.assertEquals(ws4pmSettings._getUserIdToUseInTheNameOfWith(), None)
        # now define user_mappings
        # add a lamba user to be able to test
        self.portal.acl_users.userFolderAddUser('lambda', 'lambda', ['Member'], [])
        # if the found user_mappings leads to a user that is not
        # settings.pm_username, this user mapping is returned
        settings.user_mappings = [{'local_userid': u'localUserId',
                                   'pm_userid': u'pmCreator1'},
                                  {'local_userid': u'lambda',
                                   'pm_userid': u'aUserInPloneMeeting'},
                                  {'local_userid': u'admin',
                                   'pm_userid': u'pmCreator1'}, ]
        login(self.portal, 'lambda')
        self.assertEquals(ws4pmSettings._getUserIdToUseInTheNameOfWith(), u'aUserInPloneMeeting')
        # not the user_mappings is linking to the settings.pm_username
        settings.user_mappings = [{'local_userid': u'localUserId',
                                   'pm_userid': u'pmCreator1'},
                                  {'local_userid': u'lambda',
                                   'pm_userid': u'%s' % TEST_USER_ID},
                                  {'local_userid': u'admin',
                                   'pm_userid': u'pmCreator1'}, ]
        self.assertEquals(ws4pmSettings._getUserIdToUseInTheNameOfWith(), None)
        # if the user is not the settings.pm_username and not found in the mappings
        # it is his own userId that will be used
        settings.user_mappings = [{'local_userid': u'localUserId',
                                   'pm_userid': u'pmCreator1'},
                                  {'local_userid': u'otherUser',
                                   'pm_userid': u'otherUserInPloneMeeting'},
                                  {'local_userid': u'admin',
                                   'pm_userid': u'pmCreator1'}, ]
        self.assertEquals(ws4pmSettings._getUserIdToUseInTheNameOfWith(), 'lambda')

    def test_renderTALExpression(self):
        """
          Test the method that will render a TAL expression
        """
        setRoles(self.portal, TEST_USER_ID, ('Manager',))
        login(self.portal, TEST_USER_NAME)
        # create an element to use in the TAL expression...
        document = createDocument(self.portal)
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        expr = u'python: None'
        # make sure None is never returned by the renderer as it breaks SOAP calls
        self.assertTrue(ws4pmSettings.renderTALExpression(document, self.portal, expr, {}) == u'')
        expr = u'object/Title'
        self.assertTrue(ws4pmSettings.renderTALExpression(document, self.portal, expr, {}) == u'Document title')
        expr = u'string:"My expr result"'
        self.assertTrue(ws4pmSettings.renderTALExpression(document, self.portal, expr, {}) == u'"My expr result"')
        # with a wrong expression, we raise
        expr = 'u object/wrongMethodCall'
        self.assertRaises(CompilerError, ws4pmSettings.renderTALExpression, document, self.portal, expr, {})


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(testSettings))
    return suite



# -*- coding: utf-8 -*-

from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.testing import z2
from plone.testing import zca
from Products.PloneMeeting.testing import PMLayer

import imio.pm.ws


class WSCLIENTLayer(PMLayer):
    """ """


WS4PMCLIENT_ZCML = zca.ZCMLSandbox(filename="testing.zcml",
                                   package=imio.pm.wsclient,
                                   name='WS4PMCLIENT_ZCML')

WS4PMCLIENT_Z2 = z2.IntegrationTesting(bases=(z2.STARTUP, WS4PMCLIENT_ZCML),
                                       name='WS4PMCLIENT_Z2')

WS4PMCLIENT = WSCLIENTLayer(
    zcml_filename="testing-settings.zcml",
    zcml_package=imio.pm.wsclient,
    additional_z2_products=('imio.dashboard',
                            'imio.pm.wsclient',
                            'Products.PasswordStrength'),
    gs_profile_id='imio.pm.wsclient:testing',
    name="WS4PMCLIENT")

WS4PMCLIENT_PM_TESTING_PROFILE = WSCLIENTLayer(
    zcml_filename="testing-settings.zcml",
    zcml_package=imio.pm.wsclient,
    additional_z2_products=('imio.dashboard',
                            'Products.PloneMeeting',
                            'Products.CMFPlacefulWorkflow',
                            'imio.pm.wsclient',
                            'Products.PasswordStrength'),
    gs_profile_id='imio.pm.wsclient:testing',
    name="WS4PMCLIENT_PM_TESTING_PROFILE")

WS4PMCLIENT_PM_TESTING_PROFILE_INTEGRATION = IntegrationTesting(
    bases=(WS4PMCLIENT_PM_TESTING_PROFILE,), name="WS4PMCLIENT_PM_TESTING_PROFILE_INTEGRATION")

WS4PMCLIENT_PROFILE_FUNCTIONAL = FunctionalTesting(
    bases=(WS4PMCLIENT,), name="WS4PMCLIENT_PROFILE_FUNCTIONAL")

WS4PMCLIENT_PM_TESTING_PROFILE_FUNCTIONAL = FunctionalTesting(
    bases=(
        WS4PMCLIENT_PM_TESTING_PROFILE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="WS4PMCLIENT_PM_TESTING_PROFILE_FUNCTIONAL",
)



# -*- coding: utf-8 -*-
from zope.component.interfaces import IObjectEvent
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserRequest


class IWS4PMClientLayer(IBrowserRequest):
    """
      Define a layer so the element of the WS4PM client are only available when the BrowserLayer is installed
    """


class IRedirect(Interface):
    """
    """
    def redirect():
        """
          Redirect to the right place in case we use plone.app.jquerytools overlays
        """


#
# Events interfaces
#

class IPMWSClientEvent(IObjectEvent):
    """
      All pm ws events should inherit from this class
    """


class IWillbeSendToPMEvent(IPMWSClientEvent):
    """
      An item will be send to pm.
    """


class ISentToPMEvent(IPMWSClientEvent):
    """
      An item has been sent to pm.
    """


class ISendableAnnexesToPM(Interface):
    """
    Adapts a context into a list of annexes to send to pm
    """
    def get():
        """
        Return a list of dicts representing annexes following the format:
        [
            ...,
            {
                'title': <str>,
                'UID': <UID>,
            },
            ...,
        ]
        """


class IPreferredMeetings(Interface):
    """
    Adapts a context and a list of possible meetings to filter them.
    """
    def get():
        """
        Return a list of dicts representing meeting UID's and dates following the format:
        [
            ...,
            {
                'UID': <UID>,
                'date': <str>,
            },
            ...,
        ]
        """



# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# Copyright (c) 2013 by Imio
#
# GNU General Public License (GPL)
#

import logging


logger = logging.getLogger('imio.pm.wsclient: setuphandlers')

__author__ = """Gauthier Bastien <gauthier@imio.be>"""
__docformat__ = 'plaintext'


def isNotImioPmWsClientProfile(context):
    return context.readDataFile("imio_pm_wsclient_marker.txt") is None


def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotImioPmWsClientProfile(context):
        return



# -*- coding: utf-8 -*-
#
# File: testViewlets.py
#
# GNU General Public License (GPL)
#

from Products.statusmessages.interfaces import IStatusMessage
from imio.pm.wsclient.browser.viewlets import PloneMeetingInfosViewlet
from imio.pm.wsclient.config import CAN_NOT_SEE_LINKED_ITEMS_INFO
from imio.pm.wsclient.config import CORRECTLY_SENT_TO_PM_INFO
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import cleanMemoize
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from mock import patch
from zope.annotation import IAnnotations

import transaction


class testViewlets(WS4PMCLIENTTestCase):
    """
        Tests the browser.testViewlets methods
    """

    def test_available(self):
        """ """
        self.changeUser('admin')
        document = createDocument(self.portal)
        # by default, no viewlet_display_condition TAL expression
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        settings = viewlet.ws4pmSettings.settings()
        settings.viewlet_display_condition = u''
        # no items created/linked, so the viewlet is not displayed
        self.assertFalse(viewlet.available())
        # viewlet is displayed depending on :
        # by default, the fact that it is linked to an item
        # or if a TAL expression is defined in the config and returns True
        # test with a defined TAL expression in the configuration
        settings.viewlet_display_condition = u'python: True'
        # if a TAL expression is defined, it take precedence
        # available is memoized so it is still False...
        self.assertFalse(viewlet.available())
        # remove memoized informations
        cleanMemoize(self.request, viewlet)
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        # now remove the TAL expression and send the object to PloneMeeting
        settings.viewlet_display_condition = u''
        self.assertFalse(viewlet.available())
        cleanMemoize(self.request, viewlet)
        item = self._sendToPloneMeeting(document)
        # now that the element has been sent, the viewlet is available
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        # define a TAL expression taking care of the 'isLinked'
        settings.viewlet_display_condition = u'python: isLinked and object.portal_type == "wrong_value"'
        self.assertFalse(viewlet.available())
        cleanMemoize(self.request, viewlet)
        settings.viewlet_display_condition = u'python: isLinked and object.portal_type == "Document"'
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        # if the TAL expression has errors, available is False and a message is displayed
        messages = IStatusMessage(self.request)
        # by default, 2 messages already exist, these are item creation related messages
        shownMessages = messages.show()
        self.assertTrue(len(shownMessages) == 1)
        self.assertTrue(shownMessages[0].message, CORRECTLY_SENT_TO_PM_INFO)
        settings.viewlet_display_condition = u'python: object.getUnexistingAttribute()'
        # in case there is a problem, a message is displayed in a tuple (msg, error_level)
        self.assertTrue(isinstance(viewlet.available(), tuple))
        cleanMemoize(self.request, viewlet)
        # now check when the linked item is removed
        settings.viewlet_display_condition = u''
        self.assertTrue(viewlet.available())
        cleanMemoize(self.request, viewlet)
        item.aq_inner.aq_parent.manage_delObjects(ids=[item.getId(), ])
        transaction.commit()
        self.assertFalse(viewlet.available())

    def test_getPloneMeetingLinkedInfos(self):
        """
          Test the getPloneMeetingLinkedInfos method :
          - if nothing found, an empty {} is returned
          - if elements are found, relevant data are returned
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        self.assertTrue(viewlet.available() is False)
        # now send an element to PloneMeeting and check again
        cleanMemoize(self.request, viewlet)
        item = self._sendToPloneMeeting(document)
        # we received informations about the created item
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())

    @patch("imio.pm.wsclient.browser.settings.WS4PMClientSettings._rest_searchItems")
    @patch("imio.pm.wsclient.browser.settings.WS4PMClientSettings._rest_getItemInfos")
    def test_getPloneMeetingLinkedInfos_with_linked_items(
        self, _rest_getItemInfos, _rest_searchItems
    ):
        """
        Test getPloneMeetingLinkedInfos method when there is linked items
        """
        _rest_searchItems.return_value = [{
            "@id": "http://nohost/pu-1234",
            "@type": "MeetingItemCollege",
            "UID": "1234",
            "created": "2022-01-01T00:00:00+00:00",
            "modified": "2022-01-01T00:00:00+00:00",
            "id": "pu-1234",
            "title": "PU/1244",
            "review_state": "accepted",
            "description": "Description",
            "extra_include_linked_items": [
                {
                    "@id": "http://nohost/pu-1234-1",
                    "@type": "MeetingItemCollege",
                    "UID": "2345",
                    "created": "2022-01-02T00:00:00+00:00",
                    "modified": "2022-01-02T00:00:00+00:00",
                    "id": "pu-1234-1",
                    "title": "PU/1244-1",
                    "review_state": "accepted",
                    "description": "Description",
                },
                {
                    "@id": "http://nohost/pu-1234-2",
                    "@type": "MeetingItemCollege",
                    "UID": "3456",
                    "created": "2022-01-03T00:00:00+00:00",
                    "modified": "2022-01-03T00:00:00+00:00",
                    "id": "pu-1234-2",
                    "title": "PU/1244-2",
                    "review_state": "accepted",
                    "description": "Description",
                },
            ],
            "extra_include_linked_items_items_total": "2",
        }]
        _rest_getItemInfos.side_effect = [
            [{
                "@id": "http://nohost/pu-1234",
                "@type": "MeetingItemCollege",
                "UID": "1234",
                "created": "2022-01-01T00:00:00+00:00",
                "modified": "2022-01-01T00:00:00+00:00",
                "id": "pu-1234",
                "title": "PU/1244",
                "review_state": "accepted",
                "description": "Description",
                "extra_include_config": {
                    "id": "plonemeeting-assembly",
                }
            }],
            [{
                "@id": "http://nohost/pu-1234-1",
                "@type": "MeetingItemCollege",
                "UID": "2345",
                "created": "2022-01-02T00:00:00+00:00",
                "modified": "2022-01-02T00:00:00+00:00",
                "id": "pu-1234-1",
                "title": "PU/1244-1",
                "review_state": "accepted",
                "description": "Description",
                "extra_include_config": {
                    "id": "plonemeeting-assembly",
                }
            }],
            [{
                "@id": "http://nohost/pu-1234-2",
                "@type": "MeetingItemCollege",
                "UID": "3456",
                "created": "2022-01-03T00:00:00+00:00",
                "modified": "2022-01-03T00:00:00+00:00",
                "id": "pu-1234-2",
                "title": "PU/1244-2",
                "review_state": "accepted",
                "description": "Description",
                "extra_include_config": {
                    "id": "plonegov-assembly",
                }
            }],
        ]
        self.changeUser('admin')
        document = createDocument(self.portal)
        self._sendToPloneMeeting(document)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        items = viewlet.getPloneMeetingLinkedInfos()
        self.assertEqual(3, len(items))
        self.assertEqual(
            ["pu-1234-2", "pu-1234-1", "pu-1234"],
            [e["id"] for e in items],
        )

    def test_canNotSeeLinkedInfos(self):
        """
          If the element has been sent to PloneMeeting but current user can not see these
          items in PloneMeeting, a message is displayed
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        item = self._sendToPloneMeeting(document)
        # we received informations about the created item
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())
        # admin will create the element inTheNameOf 'pmCreator1'
        self.changeUser('pmCreator1')
        cleanMemoize(self.request, viewlet)
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())
        # 'pmCreator2' will not see the infos about items, just a message
        self.changeUser('pmCreator2')
        cleanMemoize(self.request, viewlet)
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos() == (CAN_NOT_SEE_LINKED_ITEMS_INFO, 'info'))

    def test_canNotConnectTemporarily(self):
        """
          Test a connection problem in PloneMeeting after an item has already been sent
          - connect to PM successfully
          - send the item successfully
          - break settings
          - check what is going on while not being able to show PM related informations
        """
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        # send an element to PloneMeeting
        item = self._sendToPloneMeeting(document)
        # correctly sent
        self.assertTrue(viewlet.available() is True)
        self.assertTrue(viewlet.getPloneMeetingLinkedInfos()[0]['UID'] == item.UID())
        setCorrectSettingsConfig(self.portal, **{'pm_url': u'http://wrong/url'})
        cleanMemoize(self.request, viewlet)
        # no available
        # a message is returned in the viewlet by the viewlet.available method
        self.assertTrue(viewlet.available() == (UNABLE_TO_CONNECT_ERROR, 'error'))
        # the annotations on the document are still correct
        self.assertTrue(IAnnotations(document)[WS4PMCLIENT_ANNOTATION_KEY] == ['plonemeeting-assembly'])

    def test_displayMeetingDate(self):
        """
          Test the displayMeetingDate method :
          - not date, a '-' is returned
          - a date, a formatted version is returned with hours if hours != '00:00'
        """
        viewlet = PloneMeetingInfosViewlet(self.portal, self.request, None, None)
        viewlet.update()

        # The no meeting date correspond to a date in 1950
        self.assertEqual(viewlet.displayMeetingDate(""), '-')
        self.assertEqual(viewlet.displayMeetingDate(None), '-')
        self.assertEqual(viewlet.displayMeetingDate("2013-06-10"), u'2013-06-10')
        self.assertEqual(viewlet.displayMeetingDate(
            "2013-06-10 (15:30)"), u'2013-06-10 (15:30)'
        )

    def test_render(self):
        """Ensure that we can render the viewlet without any error"""
        self.changeUser('admin')
        document = createDocument(self.portal)
        viewlet = PloneMeetingInfosViewlet(document, self.request, None, None)
        viewlet.update()
        # now send an element to PloneMeeting and check again
        cleanMemoize(self.request, viewlet)
        item = self._sendToPloneMeeting(document)
        render = viewlet.render()
        self.assertTrue("Document title" in render)
        self.assertTrue("PloneMeeting assembly" in render)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testViewlets, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-

from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import ANNEXID_MANDATORY_ERROR
from imio.pm.wsclient.config import FILENAME_MANDATORY_ERROR
from imio.pm.wsclient.config import MISSING_FILE_ERROR
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import UNABLE_TO_DETECT_MIMETYPE_ERROR
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter

import base64
import logging


logger = logging.getLogger('imio.pm.wsclient')


class BaseDownloadFromItemView(BrowserView):
    def __init__(self, context, request):
        super(BrowserView, self).__init__(context, request)
        self.context = context
        self.request = request
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        self.portal = portal_state.portal()
        self.ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        self.itemUID = self.request.get('itemUID', '')

    def __call__(self):
        """ """
        # first check that we can connect to PloneMeeting
        client = self.ws4pmSettings._rest_connectToPloneMeeting()
        if not client:
            IStatusMessage(self.request).addStatusMessage(_(UNABLE_TO_CONNECT_ERROR), "error")
            return self.request.RESPONSE.redirect(self.context.absolute_url())


class GenerateItemTemplateView(BaseDownloadFromItemView):
    """
      This view manage the document generation on an item
    """
    def __init__(self, context, request):
        super(GenerateItemTemplateView, self).__init__(context, request)
        self.templateId = self.request.get('templateId', '')
        self.templateFilename = self.request.get('templateFilename', '')
        self.templateFormat = self.request.get('templateFormat', '')

    def __call__(self):
        """ """
        ws_error = super(GenerateItemTemplateView, self).__call__()
        if ws_error:
            return ws_error

        # if we can connect, proceed!
        response = self.request.RESPONSE
        mimetype = self.portal.mimetypes_registry.lookupExtension(self.templateFormat)
        if not mimetype:
            IStatusMessage(self.request).addStatusMessage(_(UNABLE_TO_DETECT_MIMETYPE_ERROR), "error")
            return response.redirect(self.context.absolute_url())

        if not self.templateFilename:
            IStatusMessage(self.request).addStatusMessage(_(FILENAME_MANDATORY_ERROR), "error")
            return response.redirect(self.context.absolute_url())

        # set relevant header for response so the browser behave normally with returned file type
        response.setHeader('Content-Type', mimetype.normalized())
        response.setHeader('Content-Disposition', 'inline;filename="%s.%s"' % (self.templateFilename,
                                                                               self.templateFormat))

        res = self.ws4pmSettings._rest_getItemTemplate({'itemUID': self.itemUID,
                                                        'templateId': self.templateId, })
        if not res:
            # an error occured, redirect to user to the context, a statusMessage will be displayed
            return self.request.RESPONSE.redirect(self.context.absolute_url())

        return res.content


class DownloadAnnexFromItemView(BaseDownloadFromItemView):
    """
      This view manage the annexes download on an item
    """

    def __init__(self, context, request):
        super(DownloadAnnexFromItemView, self).__init__(context, request)
        self.annex_id = self.request.get('annex_id', '')
        self.annex_type = self.request.get('annex_type', '')
        self.annex_type = self.annex_type and [self.annex_type] or []

    def __call__(self):
        """ """
        ws_error = super(DownloadAnnexFromItemView, self).__call__()
        if ws_error:
            return ws_error

        response = self.request.RESPONSE

        if not self.annex_id:
            IStatusMessage(self.request).addStatusMessage(_(ANNEXID_MANDATORY_ERROR), "error")
            return response.redirect(self.context.absolute_url())

        res = self.ws4pmSettings._rest_getItemInfos(
            {
                'UID': self.itemUID,
                'extra_include': 'annexes',
                'allowed_annexes_types': self.annex_type,  # XXX Must be implemented
            }
        )
        if not res:
            # an error occured, redirect to user to the context, a statusMessage will be displayed
            IStatusMessage(self.request).addStatusMessage(_(MISSING_FILE_ERROR), "error")
            return self.request.RESPONSE.redirect(self.context.absolute_url())

        annex_info = [a for a in res[0]["extra_include_annexes"] if a["id"] == self.annex_id]
        if annex_info:
            annex_info = annex_info[0]
            response.setHeader('Content-Type', annex_info["file"]["content-type"])
            response.setHeader('Content-Disposition', 'inline;filename="%s"' % annex_info["file"]["filename"])
            annex_content = self.ws4pmSettings._rest_getAnnex(annex_info["file"]["download"])
            return annex_content



# -*- coding: utf-8 -*-

from plone import api

import logging


def upgrade_to_200(context):
    logger = logging.getLogger("imio.pm.wsclient: Upgrade to REST API")
    logger.info("starting upgrade steps")
    url = api.portal.get_registry_record(
        "imio.pm.wsclient.browser.settings.IWS4PMClientSettings.pm_url",
        default=None,
    )
    if url:
        parts = url.split("ws4pm.wsdl")
        api.portal.set_registry_record(
            "imio.pm.wsclient.browser.settings.IWS4PMClientSettings.pm_url",
            parts[0],
        )
    action_generated = api.portal.get_registry_record("imio.pm.wsclient.browser.settings.IWS4PMClientSettings.generated_actions")
    for action in action_generated:
        if action["permissions"] == "SOAP Client Send":
            action["permissions"] = "WS Client Send"
        if action["permissions"] == "SOAP Client Access":
            action["permissions"] = "WS Client Access"
    api.portal.set_registry_record("imio.pm.wsclient.browser.settings.IWS4PMClientSettings.generated_actions", action_generated)
    setup = api.portal.get_tool("portal_setup")
    setup.runImportStepFromProfile('imio.pm.wsclient:default', 'rolemap')

    logger.info("upgrade step done!")



# -*- coding: utf-8 -*-
#
# File: testItemMethods.py
#
# Copyright (c) 2012 by CommunesPlone
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from AccessControl import Unauthorized
from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import ALREADY_SENT_TO_PM_ERROR
from imio.pm.wsclient.config import CORRECTLY_SENT_TO_PM_INFO
from imio.pm.wsclient.config import NO_PROPOSING_GROUP_ERROR
from imio.pm.wsclient.config import UNABLE_TO_CONNECT_ERROR
from imio.pm.wsclient.config import WS4PMCLIENT_ANNOTATION_KEY
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createAnnex
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import createDocument
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import SEND_TO_PM_VIEW_NAME
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import setCorrectSettingsConfig
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from Products.statusmessages.interfaces import IStatusMessage
from plone.app.testing import logout
from zope.annotation import IAnnotations
from zope.component import getMultiAdapter
from zope.i18n import translate

import transaction


class testForms(WS4PMCLIENTTestCase):
    """
        Tests the browser.settings SOAP client methods
    """

    def test_canNotCallFormIfNotConnected(self):
        """If no valid parameters are defined in the settings, the view is not accessible
           and a relevant message if displayed to the member in portal_messages."""
        # only available to connected users
        logout()
        self.assertRaises(Unauthorized, self.portal.restrictedTraverse, SEND_TO_PM_VIEW_NAME)
        self.changeUser('pmCreator1')
        self.portal.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance

    def test_canNotConnectToPloneMeeting(self):
        """If no valid parameters are defined in the settings, the view is not accessible
           and a relevant message if displayed to the member in portal_messages."""
        # set base params to avoid extra status messages like 'no field_mappings defined'
        setCorrectSettingsConfig(self.portal, setConnectionParams=False, withValidation=False)
        # only available to connected users
        self.changeUser('pmCreator1')
        self._configureRequestForView(self.portal)
        view = self.portal.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        # when we can not connect, a message is displayed to the user
        messages = IStatusMessage(self.request)
        self.assertTrue(len(messages.show()) == 2)
        # call the view
        view()
        self.assertTrue(len(messages.show()) == 1)
        self.assertEquals(messages.show()[0].message, UNABLE_TO_CONNECT_ERROR)

    def test_canNotExecuteWrongAction(self):
        """While calling the view to execute an action, we check if the user can actually
           execute the action regarding the parameters defined in settings.generated_actions."""
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        # build an url that should not be accessible by the user
        # if the url does not correspond to an available wsclient linked action,
        # an Unauthorized is raised.  This make sure the triggered action is
        # available to the user
        self.request.set('URL', document.absolute_url())
        self.request.set('ACTUAL_URL', document.absolute_url() + '/%s' % SEND_TO_PM_VIEW_NAME)
        self.request.set('meetingConfigId', 'wrong-meeting-config-id')
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        self.assertRaises(Unauthorized, view)

    def test_sendItemToPloneMeeting(self):
        """Test that the item is actually sent to PloneMeeting."""
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        self._configureRequestForView(document)
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        # create an annex to send...
        annex = createAnnex(self.portal.Members.pmCreator1)
        # before sending, no item is linked to the document
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        self.assertTrue(len(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})) == 0)
        # create the 'pmCreator1' member area to be able to create an item
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator1')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created member area...
        transaction.commit()
        # send to PloneMeeting
        # as form.button.Send is not in the form, nothing is done but returning the views's index
        # the form for sending an element is displayed
        form_action = '<form class="rowlike enableUnloadProtection   kssattr-formname-send_to_plonemeeting_form"' \
            ' action="{0}/Members/pmCreator1/document" method="post"' \
            ' id="form" enctype="multipart/form-data">'.format(self.portal.absolute_url())
        self.assertTrue(form_action in view())
        self.assertTrue(len(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})) == 0)
        # now send the element to PM
        view.proposingGroupId = 'developers'
        view.request.form['form.widgets.annexes'] = [annex.UID()]
        # view._doSendToPloneMeeting returns True if the element was actually sent
        self.assertTrue(view._doSendToPloneMeeting())
        # while the element is sent, the view will return nothing...
        self.assertFalse(view())
        # now that the element has been sent, an item is linked to the document
        items = ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})
        self.assertTrue(len(items) == 1)
        # moreover, as defined in the configuration, 1 annex were added to the item
        itemInfos = ws4pmSettings._rest_getItemInfos({'UID': items[0]['UID'], 'extra_include': "annexes"})[0]
        self.assertTrue(len(itemInfos['extra_include_annexes']) == 1)
        self.assertEqual("annex", itemInfos["extra_include_annexes"][0]["@type"])
        self.assertEqual(
            "annexe-oubliee.txt", itemInfos["extra_include_annexes"][0]["id"],
        )

    def test_canNotSendIfInNoPMCreatorGroup(self):
        """
          If the user that wants to send the item in PloneMeeting is not a creator in PM,
          aka is not in a _creators suffixed group, a message is displayed to him.
        """
        # remove pmCreator2 from the vendors_creators group
        # first check that the user is actually in a _creators group
        pmCreator2 = self.portal.portal_membership.getMemberById('pmCreator2')
        self.assertTrue([group for group in self.portal.acl_users.source_groups.getGroupsForPrincipal(pmCreator2)
                         if group.endswith('_creators')])
        self.portal.portal_groups.removePrincipalFromGroup('pmCreator2', self.vendors_creators)
        # pmCreator2 is no more in a _creators group
        self.assertFalse([group for group in self.portal.acl_users.source_groups.getGroupsForPrincipal(pmCreator2)
                         if group.endswith('_creators')])
        # try to send the item
        setCorrectSettingsConfig(self.portal)
        self.changeUser('pmCreator2')
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator2')
        transaction.commit()
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator2)
        messages = IStatusMessage(self.request)
        # if no item is created, _sendToPloneMeeting returns None
        self.assertFalse(self._sendToPloneMeeting(document, user='pmCreator2', proposingGroup=self.vendors_uid))
        msg = _(NO_PROPOSING_GROUP_ERROR, mapping={'userId': 'pmCreator2'})
        messages = messages.show()
        self.assertEqual(messages[-1].message, translate(msg))

    def test_checkAlreadySentToPloneMeeting(self):
        """Test in case we sent the element again to PloneMeeting, that should not happen...
           It check also that relevant annotation wipe out works correctly."""
        ws4pmSettings = getMultiAdapter((self.portal, self.request), name='ws4pmclient-settings')
        setCorrectSettingsConfig(self.portal)
        settings = ws4pmSettings.settings()
        self.changeUser('pmCreator1')
        # create an element to send...
        document = createDocument(self.portal.Members.pmCreator1)
        self._configureRequestForView(document)
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        view.proposingGroupId = 'developers'
        # create the 'pmCreator1' member area to be able to create an item
        self.tool.getPloneMeetingFolder('plonemeeting-assembly', 'pmCreator1')
        self.tool.getPloneMeetingFolder('plonegov-assembly', 'pmCreator1')
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created member area...
        transaction.commit()
        # before sending, the element is not linked
        annotations = IAnnotations(document)
        self.assertFalse(WS4PMCLIENT_ANNOTATION_KEY in annotations)
        self.assertFalse(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                                                                           self.request.get('meetingConfigId')))
        # send the document
        self.assertTrue(view._doSendToPloneMeeting())
        # is linked to one item
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == [self.request.get('meetingConfigId'), ])
        self.assertTrue(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                        (self.request.get('meetingConfigId'),)))
        self.assertTrue(len(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})) == 1)
        messages = IStatusMessage(self.request)
        # there is one message saying that the item was correctly sent
        shownMessages = messages.show()
        self.assertEquals(shownMessages[-1].message, CORRECTLY_SENT_TO_PM_INFO)
        # call form again, it will display relevant status messages
        # the rendered form is u''
        self.assertTrue(settings.only_one_sending)
        self.assertTrue(view() == u'')
        # the item is not created again
        # is still linked to one item
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == [self.request.get('meetingConfigId'), ])
        self.assertTrue(len(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})) == 1)
        # a warning is displayed to the user
        self.request.response.status = 200  # if status in 300, messages are not deleted with show
        self.assertEquals(messages.show()[-1].message, ALREADY_SENT_TO_PM_ERROR)
        settings.only_one_sending = False
        self.assertFalse(settings.only_one_sending)
        view._finishedSent = False
        self.request.response.status = 200  # if status in 300, render is not called by z3cform
        self.assertIn('form-buttons-send_to_plonemeeting', view())
        self.assertEqual(len(messages.show()), 0)
        # if we remove the item in PloneMeeting, the view is aware of it
        itemUID = str(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})[0]['UID'])
        # avoid weird ConflictError while committing because of
        # self.portal._volatile_cache_keys PersistentMapping
        self.portal._volatile_cache_keys._p_changed = False
        transaction.commit()
        item = self.portal.uid_catalog(UID=itemUID)[0].getObject()
        # remove the item
        item.aq_inner.aq_parent.manage_delObjects(ids=[item.getId(), ])
        transaction.commit()
        # checkAlreadySentToPloneMeeting will wipe out inconsistent annotations
        # for now, annotations are inconsistent
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == [self.request.get('meetingConfigId'), ])
        self.assertFalse(
            view.ws4pmSettings.checkAlreadySentToPloneMeeting(
                document,
                (self.request.get('meetingConfigId'),)))
        # now it is consistent
        self.assertFalse(WS4PMCLIENT_ANNOTATION_KEY in annotations)
        self.assertTrue(len(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})) == 0)
        # the item can be sent again and will be linked to a new created item
        self.assertTrue(view._doSendToPloneMeeting())
        self.assertTrue(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                        (self.request.get('meetingConfigId'),)))
        self.assertTrue(len(ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})) == 1)
        # the item can also been send to another meetingConfig
        self.request.set('meetingConfigId', 'plonegov-assembly')
        view = document.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        view.proposingGroupId = 'developers'
        self.assertFalse(
            view.ws4pmSettings.checkAlreadySentToPloneMeeting(
                document,
                (self.request.get('meetingConfigId'),)))
        self.request.form['form.widgets.category'] = [u'deployment', ]
        self.assertTrue(view._doSendToPloneMeeting())
        self.assertTrue(view.ws4pmSettings.checkAlreadySentToPloneMeeting(document,
                        (self.request.get('meetingConfigId'),)))
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == ['plonemeeting-assembly',
                                                                    self.request.get('meetingConfigId'), ])
        # if we remove the 2 items, a call to checkAlreadySentToPloneMeeting
        # without meetingConfigs will wipeout the annotations
        transaction.commit()
        itemUIDs = [str(elt['UID']) for elt in ws4pmSettings._rest_searchItems({'externalIdentifier': document.UID()})]

        item1 = self.portal.uid_catalog(UID=itemUIDs[0])[0].getObject()
        item2 = self.portal.uid_catalog(UID=itemUIDs[1])[0].getObject()
        item1.aq_inner.aq_parent.manage_delObjects(ids=[item1.getId(), ])
        item2.aq_inner.aq_parent.manage_delObjects(ids=[item2.getId(), ])
        transaction.commit()
        # annotations are still messed up
        self.assertTrue(annotations[WS4PMCLIENT_ANNOTATION_KEY] == ['plonemeeting-assembly',
                                                                    self.request.get('meetingConfigId'), ])
        # wipe out annotations
        view.ws4pmSettings.checkAlreadySentToPloneMeeting(document)
        self.assertFalse(WS4PMCLIENT_ANNOTATION_KEY in annotations)

    def _configureRequestForView(self, obj):
        """Helper method that configure the request values so the view to test works..."""
        self.request.set('URL', obj.absolute_url())
        self.request.set('ACTUAL_URL', obj.absolute_url() + '/%s' % SEND_TO_PM_VIEW_NAME)
        self.request.set('meetingConfigId', 'plonemeeting-assembly')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testForms, prefix='test_'))
    return suite



# -*- coding: utf-8 -*-

from imio.pm.wsclient.interfaces import IPMWSClientEvent
from imio.pm.wsclient.interfaces import ISentToPMEvent
from imio.pm.wsclient.interfaces import IWillbeSendToPMEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class PMWSClientEvent(ObjectEvent):
    """
      Abstract pm ws event. All pm ws events should inherit from it
    """
    implements(IPMWSClientEvent)


class WillbeSendToPMEvent(PMWSClientEvent):
    """
      Notified when an item is about to be sent to PM.
    """
    implements(IWillbeSendToPMEvent)


class SentToPMEvent(PMWSClientEvent):
    """
       Notified when an item has been successfully sent to PM.
    """
    implements(ISentToPMEvent)



# -*- coding: utf-8 -*-

from zope.i18nmessageid import MessageFactory


WS4PMClientMessageFactory = MessageFactory("imio.pm.wsclient")
PMMessageFactory = MessageFactory("PloneMeeting")


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# -*- coding: utf-8 -*-

from imio.pm.wsclient.browser import vocabularies
from imio.pm.wsclient.tests.WS4PMCLIENTTestCase import WS4PMCLIENTTestCase
from mock import patch
from plone import api
from WS4PMCLIENTTestCase import setCorrectSettingsConfig
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest
from zope.globalrequest import setRequest


class testVocabularies(WS4PMCLIENTTestCase):
    """
    Test the vocabularies.
    """

    @patch("imio.pm.wsclient.browser.settings.WS4PMClientSettings._rest_getConfigInfos")
    def test_pm_meeting_config_id_vocabulary(self, _rest_getConfigInfos):
        """ """
        _rest_getConfigInfos.return_value = [
            {"id": u"plonegov-assembly", "title": u"PloneGov Assembly"},
            {"id": u"plonemeeting-assembly", "title": u"PloneMeeting Assembly"},
        ]
        raw_voc = vocabularies.pm_meeting_config_id_vocabularyFactory()
        self.assertEqual(len(raw_voc), 2)
        voc = [v for v in raw_voc]
        self.assertEqual(voc[0].value, "plonegov-assembly")
        self.assertEqual(voc[1].value, "plonemeeting-assembly")

    @patch("imio.pm.wsclient.browser.settings.WS4PMClientSettings._rest_getConfigInfos")
    @patch("imio.pm.wsclient.browser.settings.WS4PMClientSettings._rest_getMeetingsAcceptingItems")  # noqa
    def test_desired_meetingdates_vocabulary(self, _rest_getMeetingsAcceptingItems, _rest_getConfigInfos):
        """Ensure that vocabularies values are the expected"""
        _rest_getConfigInfos.return_value = [
            {"id": u"plonegov-assembly", "title": u"PloneGov Assembly"},
            {"id": u"plonemeeting-assembly", "title": u"PloneMeeting Assembly"},
        ]

        request = getRequest()
        request.set("meetingConfigId", u"plonemeeting-assembly")
        setRequest(request)
        _rest_getMeetingsAcceptingItems.return_value = [
            {
                u"@extra_includes": [],
                u"@id": u"http://localhost:63033/plone/Members/pmManager/mymeetings/plonemeeting-assembly/o2",  # noqa
                u"@type": u"MeetingPma",
                u"UID": u"96b61ee6cc6d46b6a84c207d89e8ea51",
                u"created": u"2022-07-13T11:47:34+00:00",
                u"description": u"",
                u"enabled": None,
                u"id": u"o2",
                u"modified": u"2022-07-13T11:47:35+00:00",
                u"review_state": u"created",
                u"title": u"03 march 2013",
                u"date": u"2013-03-03T00:00:00",
            },
            {
                u"@extra_includes": [],
                u"@id": u"http://localhost:63033/plone/Members/pmManager/mymeetings/plonemeeting-assembly/o1",  # noqa
                u"@type": u"MeetingPma",
                u"UID": u"d7de4c8580bb404fbfbd5ea5963e9c64",
                u"created": u"2022-07-13T11:47:33+00:00",
                u"description": u"",
                u"enabled": None,
                u"id": u"o1",
                u"modified": u"2022-07-13T11:47:34+00:00",
                u"review_state": u"created",
                u"title": u"03 march 2013",
                u"date": u"2013-03-03T00:00:00",
            },
        ]
        raw_voc = vocabularies.desired_meetingdates_vocabularyFactory(api.portal.get())
        self.assertEqual(len(raw_voc), 2)

        # Test that the cache is used if the request did not change
        # This example should not happen in real life, but it is a good test
        _rest_getMeetingsAcceptingItems.return_value = [
            {
                u"@extra_includes": [],
                u"@id": u"http://localhost:63033/plone/Members/pmManager/mymeetings/plonegov-assembly/o1",  # noqa
                u"@type": u"MeetingPma",
                u"UID": u"89ada78808d04a04b145518b2a469f2a",
                u"created": u"2022-08-13T11:47:33+00:00",
                u"description": u"",
                u"enabled": None,
                u"id": u"o1",
                u"modified": u"2022-08-13T11:47:34+00:00",
                u"review_state": u"created",
                u"title": u"03 march 2013",
                u"date": u"2013-08-03T00:00:00",
            },
        ]
        raw_voc = vocabularies.desired_meetingdates_vocabularyFactory(api.portal.get())
        self.assertEqual(len(raw_voc), 2)

        # Test cache is not used if the request changed
        request = getRequest()
        request.set("meetingConfigId", u"plonegov-assembly")
        setRequest(request)
        raw_voc = vocabularies.desired_meetingdates_vocabularyFactory(api.portal.get())
        self.assertEqual(len(raw_voc), 1)
        self.assertEqual(list(raw_voc)[0].title, u"03/08/2013 00:00")
        self.assertEqual(list(raw_voc)[0].value, "89ada78808d04a04b145518b2a469f2a")
        self.assertEqual(list(raw_voc)[0].token, "89ada78808d04a04b145518b2a469f2a")

    @patch("imio.pm.wsclient.browser.settings.WS4PMClientSettings._rest_getConfigInfos")
    def test_categories_for_user_vocabulary(self, _rest_getConfigInfos):
        """Ensure that vocabularies values are the expected"""

        # Set the correct settings
        setCorrectSettingsConfig(self.portal, setConnectionParams=False, withValidation=False)
        ws4pmsettings = getMultiAdapter((self.portal, self.portal.REQUEST), name="ws4pmclient-settings")
        for i in range(len(ws4pmsettings.settings().field_mappings)):
            if ws4pmsettings.settings().field_mappings[i]["field_name"] == "category":
                del ws4pmsettings.settings().field_mappings[i]
                break
        self.portal.REQUEST.set("meetingConfigId", u"plonegov-assembly")

        # First test with categories
        _rest_getConfigInfos.return_value = [
            {
                "id": u"plonegov-assembly",
                "title": u"PloneGov Assembly",
                "categories": [
                    {
                        "id": "urbanisme",
                        "title": "Urbanisme",
                        "enabled": True,
                    },
                    {
                        "id": "finances",
                        "title": "Finances",
                        "enabled": True,
                    },
                ],
            },
            {"id": u"plonemeeting-assembly", "title": u"PloneMeeting Assembly"},
        ]
        voc = vocabularies.categories_for_user_vocabulary()(self.portal)
        self.assertListEqual(
            [(v.token, v.title) for v in voc], [("finances", u"Finances"), ("urbanisme", u"Urbanisme")]
        )

        # Second test without categories
        _rest_getConfigInfos.return_value = []
        voc = vocabularies.categories_for_user_vocabulary()(self.portal)
        self.assertListEqual(list(voc), [])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from Acquisition import aq_base
from plonemeeting.restapi.tests.base import BaseTestCase as WS4PMTestCase
from imio.pm.wsclient.testing import WS4PMCLIENT_PM_TESTING_PROFILE_FUNCTIONAL
from Products.PloneMeeting.config import DEFAULT_USER_PASSWORD
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter

import transaction


SEND_TO_PM_VIEW_NAME = '@@send_to_plonemeeting_form'


class WS4PMCLIENTTestCase(WS4PMTestCase):
    '''Base class for defining WS4PMCLIENT test cases.'''

    # define PM base TestCase test file that we will not launch from here
    subproductIgnoredTestFiles = ['testMeetingCategory.py', 'testVotes.py', 'testWFAdaptations.py',
                                  'testPodTemplates.py', 'testAdvices.py', 'testMeetingFileType.py',
                                  'testMeeting.py', 'testChangeItemOrderView.py', 'testMeetingGroup.py',
                                  'testWorkflows.py', 'testMeetingConfig.py', 'testToolPloneMeeting.py',
                                  'testPortlets.py', 'testMeetingItem.py', 'testConversionWithDocumentViewer.py',
                                  'testPerformances.py']

    layer = WS4PMCLIENT_PM_TESTING_PROFILE_FUNCTIONAL

    def setUp(self):
        """ """
        WS4PMTestCase.setUp(self)

    def _sendToPloneMeeting(self, obj, user='pmCreator1', proposingGroup='developers',
                            meetingConfigId='plonemeeting-assembly', category=''):
        """
          Helper method for sending an element to PloneMeeting
        """
        # set correct config
        setCorrectSettingsConfig(self.portal)
        # create the 'pmCreator1' member area to be able to create an item
        self.tool.getPloneMeetingFolder(meetingConfigId, user)
        # we have to commit() here or portal used behing the SOAP call
        # does not have the freshly created item...
        transaction.commit()
        # use the 'send_to_plonemeeting' view
        self.request.set('URL', obj.absolute_url())
        self.request.set('ACTUAL_URL', obj.absolute_url() + '/%s' % SEND_TO_PM_VIEW_NAME)
        self.request.set('meetingConfigId', meetingConfigId)
        view = obj.restrictedTraverse(SEND_TO_PM_VIEW_NAME).form_instance
        view.proposingGroupId = proposingGroup
        # call the view so relevant status messages are displayed
        view()
        # we start a new transaction because view._doSendToPloneMeeting creates the
        # item using a suds SOAP subprocess call and the item is actually created when the transaction is committed
        # if not, the item is not really created and not found here under by the portal_catalog query
        transaction.begin()
        view._doSendToPloneMeeting()
        transaction.commit()
        brains = self.portal.portal_catalog(portal_type='MeetingItemPma', externalIdentifier=obj.UID())
        return brains and brains[0].getObject() or None


def setCorrectSettingsConfig(portal, setConnectionParams=True, minimal=False, withValidation=True, **kwargs):
    """Set a workable set of settings for tests.
       If p_withValidation is False, remove validation because we want
       to force to set some values and relevant vocabularies for example do not contain that value.
       If p_minimal is True, only minimal settings are set to be able to connect."""
    if not withValidation:
        # disable validation when forcing some values to set
        from zope.schema._field import AbstractCollection
        old_validate = AbstractCollection._validate

        def _validate(self, value):
            return
        AbstractCollection._validate = _validate
    ws4pmSettings = getMultiAdapter((portal, portal.REQUEST), name='ws4pmclient-settings')
    settings = ws4pmSettings.settings()
    if setConnectionParams:
        settings.pm_url = kwargs.get('pm_url', None) or portal.absolute_url().decode('utf-8')
        settings.pm_username = kwargs.get('pm_username', None) or u'pmManager'
        settings.pm_password = kwargs.get('pm_password', None) or DEFAULT_USER_PASSWORD
    settings.user_mappings = kwargs.get('user_mappings', None) or \
        [{'local_userid': u'localUserId',
          'pm_userid': u'pmCreator1'},
         {'local_userid': u'localUserId2',
          'pm_userid': u'pmCreator2'},
         {'local_userid': u'admin',
          'pm_userid': u'pmCreator1'}, ]
    settings.viewlet_display_condition = kwargs.get('viewlet_display_condition', None) or u''
    if not minimal:
        # these parameters are only available while correctly connected
        # to PloneMeeting webservices, either use withValidation=False
        # these fields mappings make it work if classic Document content_type
        settings.field_mappings = kwargs.get('field_mappings', None) or [
            {'field_name': u'title',
             'expression': u'object/Title'},
            {'field_name': u'description',
             'expression': u'object/Description'},
            # 'plonemeeting-assembly' does not use categories but 'plonegov-assembly' does
            {'field_name': u'category',
             'expression':
                u'python: object.REQUEST.get("meetingConfigId") != "plonemeeting-assembly" and "deployment" or ""'},
            {'field_name': u'decision',
             'expression': u'object/getText'},
            {'field_name': u'externalIdentifier',
             'expression': u'object/UID'},
        ]
        settings.allowed_annexes_types = kwargs.get('allowed_annexes_types', None) or []
        settings.generated_actions = kwargs.get('generated_actions', None) or [
            {'pm_meeting_config_id': 'plonegov-assembly',
             'condition': u'python:True',
             'permissions': u'View'},
            {'pm_meeting_config_id': 'plonegov-assembly',
             'condition': u'python:True',
             'permissions': u'View'},
            {'pm_meeting_config_id': 'plonemeeting-assembly',
             'condition': u'python:True',
             'permissions': u'View'},
            {'pm_meeting_config_id': 'plonemeeting-assembly',
             'condition': u'python:False',
             'permissions': u'View'},
            {'pm_meeting_config_id': 'plonemeeting-assembly',
             'condition': u'python:True',
             'permissions': u'Manage portal'}

        ]
    if not withValidation:
        AbstractCollection._validate = old_validate


def createDocument(placeToCreate):
    """
      Helper method for creating a document object
    """
    data = {'title': 'Document title',
            'description': 'Document description',
            'text': '<p>Document rich text</p>'}
    documentId = placeToCreate.invokeFactory('Document', id='document', **data)
    document = getattr(placeToCreate, documentId)
    document.reindexObject()
    return document


def createAnnex(placeToCreate):
    """
      Helper method for creating an annex object
    """
    data = {'title': 'Annexe oubliée',
            'file': 'hello\n'}
    annexId = placeToCreate.invokeFactory('File', id='annex', **data)
    annex = getattr(placeToCreate, annexId)
    annex.getFile().setFilename('annexe oubliée.txt')
    annex.reindexObject()
    return annex


def cleanMemoize(request, obj=None):
    """
      Remove every memoized informations : memoize on the REQUEST and on the object
    """
    annotations = IAnnotations(request)
    if 'plone.memoize' in annotations:
        annotations['plone.memoize'].clear()
    if obj and hasattr(aq_base(obj), '_memojito_'):
        delattr(obj, '_memojito_')



from setuptools import setup, find_packages
import os

version = '2.0.4'

setup(name='imio.pm.wsclient',
      version=version,
      description="WebServices Client for PloneMeeting",
      long_description=open("README.txt").read() + "\n\n" +
                       open(os.path.join("CHANGES.rst")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        ],
      keywords='',
      author='Gauthier Bastien',
      author_email='devs@imio.be',
      url='https://github.com/IMIO/imio.pm.wsclient',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['imio', 'imio.pm'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'collective.z3cform.datagridfield',
          'imio.pm.locales',
          'plone.memoize',
          'requests',
          'six',
      ],
      extras_require={'test': ['plone.app.testing', 'plonemeeting.restapi', 'mock']},
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from imio.pm.wsclient import WS4PMClientMessageFactory as _

# suffix used when adding our actions to portal_actions
ACTION_SUFFIX = 'plonemeeting_wsclient_action_'

# taken from imio.pm.ws
DEFAULT_NO_WARNING_MESSAGE = _('There was NO WARNING message during item creation.')

# messages
CAN_NOT_SEE_LINKED_ITEMS_INFO = _(u"This element is linked to item(s) in PloneMeeting but your are not allowed to see it.")
CORRECTLY_SENT_TO_PM_INFO = _(u"The item has been correctly sent to PloneMeeting.")
CONFIG_UNABLE_TO_CONNECT_ERROR = _(u"Unable to connect to PloneMeeting! The error message was : ${error}!")
UNABLE_TO_CONNECT_ERROR = _(u"Unable to connect to PloneMeeting! Please contact system administrator!")
ALREADY_SENT_TO_PM_ERROR = _(u"This element has already been sent to PloneMeeting!")
TAL_EVAL_FIELD_ERROR = _(
    u"There was an error evaluating the TAL expression '${expr}' for the field '${field_name}'! "
    "The error was : ${error}. Please contact system administrator."
)
UNABLE_TO_DETECT_MIMETYPE_ERROR = _(
    u"Could not detect correct mimetype for item template! "
    "Please contact system administrator!"
)
FILENAME_MANDATORY_ERROR = _(
    u"A filename is mandatory while generating a document! "
    "Please contact system administrator!"
)
UNABLE_TO_DISPLAY_VIEWLET_ERROR = _(
    u"Unable to display informations about the potentially linked item "
    "in PloneMeeting because there was an error evaluating the TAL expression '${expr}' "
    "for the field '${field_name}'! The error was : '${error}'.  Please contact system administrator."
)
CONFIG_CREATE_ITEM_PM_ERROR = _(
    u"An error occured during the item creation in PloneMeeting! "
    "The error message was : ${error}"
)
NO_PROPOSING_GROUP_ERROR = _(
    u"An error occured during the item creation in PloneMeeting! The error message was "
    u": [{'field': 'proposingGroup', 'message': u'Proposing group is not available for "
    u"user.', 'error': 'ValidationError'}]"
)
NO_USER_INFOS_ERROR = _(u"Could not get userInfos in PloneMeeting for user '${userId}'!")
NO_FIELD_MAPPINGS_ERROR = _(u"No field_mappings defined in the WS4PMClient configuration!")
CAN_NOT_CREATE_FOR_PROPOSING_GROUP_ERROR = _(
    u"The current user can not create an item with the proposingGroup forced "
    "thru the configuration! Please contact system administrator!"
)
NO_CONFIG_INFOS_ERROR = _(u"No configuration informations found!")
CAN_NOT_CREATE_WITH_CATEGORY_ERROR = _(
    u"The current user can not create an item with the category forced "
    "thru the configuration! Please contact system administrator!"
)
SEND_WITHOUT_SUFFICIENT_FIELD_MAPPINGS_DEFINED_WARNING = _(
    u"No sufficient field mappings are defined in the "
    "configuration. It is recommended to define at least the "
    "'title' mapping, but 'description' and 'decision' should "
    "also be defined. It will ne be possible to create the "
    "item in PloneMeeting."
)
ANNEXID_MANDATORY_ERROR = _(u"An annex id is mandatory to download an annex!")
MISSING_FILE_ERROR = _(u"The requested file could not be found on the item")

# annotations key
WS4PMCLIENT_ANNOTATION_KEY = "imio.pm.wsclient-sent_to"



# -*- coding: utf-8 -*-

from datetime import datetime
from imio.pm.wsclient import WS4PMClientMessageFactory as _
from imio.pm.wsclient.config import CAN_NOT_CREATE_FOR_PROPOSING_GROUP_ERROR
from imio.pm.wsclient.config import CAN_NOT_CREATE_WITH_CATEGORY_ERROR
from imio.pm.wsclient.config import NO_CONFIG_INFOS_ERROR
from imio.pm.wsclient.config import NO_FIELD_MAPPINGS_ERROR
from imio.pm.wsclient.config import NO_USER_INFOS_ERROR
from imio.pm.wsclient.config import TAL_EVAL_FIELD_ERROR
from imio.pm.wsclient.interfaces import IPreferredMeetings
from imio.pm.wsclient.interfaces import ISendableAnnexesToPM
from natsort import humansorted
from operator import itemgetter
from plone import api
from plone.memoize import ram
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter
from zope.component import queryAdapter
from zope.component import queryMultiAdapter
from zope.component.hooks import getSite
from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import pytz


class pm_meeting_config_id_vocabulary(object):

    implements(IVocabularyFactory)

    def __call__(self, context=None):
        """Query every existing MeetingConfigs in a distant PloneMeeting."""
        # query existing MeetingGroups from distant PM site if the default_pm_url is defined and working
        portal = getSite()
        # while called in an inline_validation, portal is not correct...
        if not portal.__module__ == "Products.CMFPlone.Portal":
            portal = portal.aq_inner.aq_parent
        settings = getMultiAdapter((portal, portal.REQUEST), name="ws4pmclient-settings")
        pmConfigInfos = settings._rest_getConfigInfos()
        terms = []
        if pmConfigInfos:
            for pmConfigInfo in pmConfigInfos:
                terms.append(
                    SimpleTerm(
                        unicode(pmConfigInfo["id"]),
                        unicode(pmConfigInfo["id"]),
                        unicode(pmConfigInfo["title"]),
                    )
                )
        return SimpleVocabulary(terms)


pm_meeting_config_id_vocabularyFactory = pm_meeting_config_id_vocabulary()


class possible_permissions_vocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Query every existing permissions."""
        terms = []
        portal = getSite()
        # while called in an inline_validation, portal is not correct...
        if not portal.__module__ == "Products.CMFPlone.Portal":
            portal = portal.aq_inner.aq_parent
        for possible_permission in portal.acl_users.portal_role_manager.possible_permissions():
            terms.append(SimpleTerm(possible_permission, possible_permission, possible_permission))
        return SimpleVocabulary(terms)


possible_permissions_vocabularyFactory = possible_permissions_vocabulary()


class pm_item_data_vocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Query every available data we can use to create an item in the distant PloneMeeting."""
        terms = []

        # query existing item data from distant PM site if the default_pm_url is defined and working
        portal = getSite()
        # while called in an inline_validation, portal is not correct...
        if not portal.__module__ == "Products.CMFPlone.Portal":
            portal = portal.aq_inner.aq_parent
        settings = getMultiAdapter((portal, portal.REQUEST), name="ws4pmclient-settings")
        availableDatas = settings._rest_getItemCreationAvailableData()
        if availableDatas:
            for availableData in availableDatas:
                terms.append(
                    SimpleTerm(
                        unicode(availableData),
                        unicode(availableData),
                        unicode(availableData),
                    )
                )
        return SimpleVocabulary(terms)


pm_item_data_vocabularyFactory = pm_item_data_vocabulary()


class proposing_groups_for_user_vocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Query every available proposingGroups for current user in a distant PloneMeeting."""
        portal = getSite()
        # while called in an inline_validation, portal is not correct...
        if not portal.__module__ == "Products.CMFPlone.Portal":
            portal = portal.aq_inner.aq_parent
        ws4pmsettings = getMultiAdapter((portal, portal.REQUEST), name="ws4pmclient-settings")
        field_mappings = ws4pmsettings.settings().field_mappings
        if not field_mappings:
            portal.REQUEST.set("error_in_vocabularies", True)
            IStatusMessage(portal.REQUEST).addStatusMessage(_(NO_FIELD_MAPPINGS_ERROR), "error")
            return SimpleVocabulary([])
        forcedProposingGroup = None
        vars = {}
        vars["meetingConfigId"] = portal.REQUEST.get("meetingConfigId")
        for field_mapping in field_mappings:
            # try to find out if a proposingGroup is forced in the configuration
            if field_mapping[u"field_name"] == "proposingGroup":
                try:
                    forcedProposingGroup = ws4pmsettings.renderTALExpression(
                        context, portal, field_mapping["expression"], vars
                    )
                    break
                except Exception, e:
                    portal.REQUEST.set("error_in_vocabularies", True)
                    IStatusMessage(portal.REQUEST).addStatusMessage(
                        _(
                            TAL_EVAL_FIELD_ERROR,
                            mapping={
                                "expr": field_mapping["expression"],
                                "field_name": field_mapping["field_name"],
                                "error": e,
                            },
                        ),
                        "error",
                    )
                    return SimpleVocabulary([])
        # even if we get a forcedProposingGroup, double check that the current user can actually use it
        userInfos = ws4pmsettings._rest_getUserInfos(showGroups=True, suffix="creators")
        if not userInfos or "extra_include_groups" not in userInfos:
            portal.REQUEST.set("error_in_vocabularies", True)
            # add a status message if the main error is not the fact that we can not connect to the WS
            if userInfos is not None:
                userThatWillCreate = ws4pmsettings._getUserIdToUseInTheNameOfWith()
                IStatusMessage(portal.REQUEST).addStatusMessage(
                    _(NO_USER_INFOS_ERROR, mapping={"userId": userThatWillCreate}), "error"
                )
            return SimpleVocabulary([])
        terms = []
        forcedProposingGroupExists = not forcedProposingGroup and True or False
        for group in humansorted(userInfos["extra_include_groups"], key=lambda x: itemgetter("title")(x)):
            if forcedProposingGroup == group["id"]:
                forcedProposingGroupExists = True
                terms.append(
                    SimpleTerm(
                        unicode(group["UID"]),
                        unicode(group["UID"]),
                        unicode(group["title"]),
                    )
                )
                break
            if not forcedProposingGroup:
                terms.append(
                    SimpleTerm(
                        unicode(group["UID"]),
                        unicode(group["UID"]),
                        unicode(group["title"]),
                    )
                )
        if not forcedProposingGroupExists:
            portal.REQUEST.set("error_in_vocabularies", True)
            IStatusMessage(portal.REQUEST).addStatusMessage(_(CAN_NOT_CREATE_FOR_PROPOSING_GROUP_ERROR), "error")
            return SimpleVocabulary([])
        return SimpleVocabulary(terms)


proposing_groups_for_user_vocabularyFactory = proposing_groups_for_user_vocabulary()


class categories_for_user_vocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Query every available categories for current user in a distant PloneMeeting."""
        portal = getSite()
        # while called in an inline_validation, portal is not correct...
        if not portal.__module__ == "Products.CMFPlone.Portal":
            portal = portal.aq_inner.aq_parent
        ws4pmsettings = getMultiAdapter((portal, portal.REQUEST), name="ws4pmclient-settings")
        field_mappings = ws4pmsettings.settings().field_mappings
        if not field_mappings:
            portal.REQUEST.set("error_in_vocabularies", True)
            IStatusMessage(portal.REQUEST).addStatusMessage(_(NO_FIELD_MAPPINGS_ERROR), "error")
            return SimpleVocabulary([])
        forcedCategory = None
        vars = {}
        meetingConfigId = portal.REQUEST.get("meetingConfigId") or portal.REQUEST.form.get(
            "form.widgets.meetingConfigId"
        )
        vars["meetingConfigId"] = meetingConfigId
        for field_mapping in field_mappings:
            # try to find out if a proposingGroup is forced in the configuration
            if field_mapping[u"field_name"] == "category":
                try:
                    forcedCategory = ws4pmsettings.renderTALExpression(
                        context, portal, field_mapping["expression"], vars
                    )
                    break
                except Exception, e:
                    portal.REQUEST.set("error_in_vocabularies", True)
                    IStatusMessage(portal.REQUEST).addStatusMessage(
                        _(
                            TAL_EVAL_FIELD_ERROR,
                            mapping={
                                "expr": field_mapping["expression"],
                                "field_name": field_mapping["field_name"],
                                "error": e,
                            },
                        ),
                        "error",
                    )
                    return SimpleVocabulary([])

        configInfos = ws4pmsettings._rest_getConfigInfos(showCategories=True)
        if not configInfos:
            portal.REQUEST.set("error_in_vocabularies", True)
            # add a status message if the main error is not the fact that we can not connect to the WS
            if configInfos is not None:
                IStatusMessage(portal.REQUEST).addStatusMessage(_(NO_CONFIG_INFOS_ERROR), "error")
            return SimpleVocabulary([])
        categories = []
        # find categories for given meetingConfigId
        for configInfo in configInfos:
            if configInfo["id"] == meetingConfigId:
                categories = configInfo.get("categories", ())
                break
        # if not categories is returned, it means that the meetingConfig does
        # not use categories...
        if not categories:
            return SimpleVocabulary([])
        terms = []
        forcedCategoryExists = not forcedCategory and True or False
        for category in humansorted(categories, key=lambda x: itemgetter("title")(x)):
            if forcedCategory == category["id"]:
                forcedCategoryExists = True
                terms.append(
                    SimpleTerm(
                        unicode(category["id"]),
                        unicode(category["id"]),
                        unicode(category["title"]),
                    )
                )
                break
            if not forcedCategory:
                terms.append(
                    SimpleTerm(
                        unicode(category["id"]),
                        unicode(category["id"]),
                        unicode(category["title"]),
                    )
                )
        if not forcedCategoryExists:
            portal.REQUEST.set("error_in_vocabularies", True)
            IStatusMessage(portal.REQUEST).addStatusMessage(_(CAN_NOT_CREATE_WITH_CATEGORY_ERROR), "error")
            return SimpleVocabulary([])
        return SimpleVocabulary(terms)


categories_for_user_vocabularyFactory = categories_for_user_vocabulary()


def desired_meetingdates_vocabulary__call___cachekey(method, self, context):
    """Cache key for desired_meetingdates_vocabulary.__call__, valid forever."""
    request = api.portal.getRequest()
    meetingConfigId = request.get('meetingConfigId', request.form.get('form.widgets.meetingConfigId'))
    return meetingConfigId


class desired_meetingdates_vocabulary(object):
    implements(IVocabularyFactory)

    @ram.cache(desired_meetingdates_vocabulary__call___cachekey)
    def __call__(self, context):
        """Query every available categories for current user in a distant PloneMeeting."""
        portal = getSite()
        # while called in an inline_validation, portal is not correct...
        if not portal.__module__ == "Products.CMFPlone.Portal":
            portal = portal.aq_inner.aq_parent
        ws4pmsettings = getMultiAdapter((portal, portal.REQUEST), name="ws4pmclient-settings")
        configInfos = ws4pmsettings._rest_getConfigInfos(showCategories=True)
        if not configInfos:
            portal.REQUEST.set("error_in_vocabularies", True)
            # add a status message if the main error is not the fact that we can not connect to the WS
            if configInfos is not None:
                IStatusMessage(portal.REQUEST).addStatusMessage(_(NO_CONFIG_INFOS_ERROR), "error")
            return SimpleVocabulary([])
        request = api.portal.getRequest()
        meeting_config_id = request.get("meetingConfigId", request.form.get("form.widgets.meetingConfigId"))
        data = {"meetingConfigId": meeting_config_id}
        possible_meetings = ws4pmsettings._rest_getMeetingsAcceptingItems(data)
        local = pytz.timezone("Europe/Brussels")
        if not possible_meetings:
            return SimpleVocabulary([])
        for meeting in possible_meetings:
            meeting["date"] = datetime.strptime(meeting["date"], "%Y-%m-%dT%H:%M:%S")
            meeting["date"] = local.localize(meeting["date"])
            meeting['date'] = meeting['date'].astimezone(local)
        terms = []
        allowed_meetings = queryMultiAdapter((context, possible_meetings), IPreferredMeetings)
        meetings = allowed_meetings and allowed_meetings.get() or possible_meetings
        for meeting_info in meetings:
            display_date = datetime.strftime(meeting_info["date"], "%d/%m/%Y %H:%M") \
                if isinstance(meeting_info["date"], datetime) else meeting_info["date"]
            terms.append(
                SimpleTerm(
                    unicode(meeting_info["UID"]),
                    unicode(meeting_info["UID"]),
                    unicode(display_date),
                )
            )
        return SimpleVocabulary(terms)


desired_meetingdates_vocabularyFactory = desired_meetingdates_vocabulary()


class annexes_for_user_vocabulary(object):
    implements(IVocabularyFactory)

    def __call__(self, context):
        """Query every available data we can use to create an item in the distant PloneMeeting."""
        terms = []
        to_annexes = queryAdapter(context, ISendableAnnexesToPM)

        if to_annexes:
            for annex in to_annexes.get():
                terms.append(
                    SimpleTerm(
                        annex["UID"],
                        annex["UID"],
                        annex["title"],
                    )
                )
        return SimpleVocabulary(terms)


annexes_for_user_vocabularyFactory = annexes_for_user_vocabulary()
